.pagelength 58t		; directive tells assembler to make pages 58 lines
*
*************************************************
*			    CHARGER	Version O
*
* 10 MAY 93 - BEGAN MAXIMUM VOLTAGE CHARGE COMPLETE BASED ON BATTERY TEMP.
*	1C - SOFTWARE VERSION
* 12 MAY 93 - DECREASED PUMP TIME TO 12 MINUTES
*	1D - SOFTWARE VERSION
* 18 MAY 93 - INCREASED PUMP TIME TO 26 MINUTES
*	1E - SOFTWARE VERSION
* 14  JUN 93 - ELIMINATED CHARGE CYCLING IN SINGLE PHASE MODE.	CURRENT MUST
*		BE AT LEAST 16 AMPS TO REACH FULL CHARGE. CHARGES AT MAXIMUM
*		SINGLE PHASE CURRENT (20 AMPS) FOR ENTIRE CHARGE.
*	     -	MAXIMUM AMP*HOUR CUT-BACK CORRECTED FOR SINGLE PHASE
*
* 16 JUN 93 - CHARGE CURRENT DECREASED IF FUSE IS LOST ON 3 PHASE
*
* 17 JUN 93 - CLEANED UP PROGRAM FLOW AND REDUCED CODE.
*	      NO WATER CYCLE IF TOTAL AMP*HOUR CHARGE IS LESS THAN 100 A*HR
*
* 28 JUN 93 - MODIFIED FINAL CHARGE CURRENT AS A FUNCTION OF BATT TEMP. AS
*		TEMP INCREASES, CHARGE CURRENT INCREASES SO THAT A FINAL
*		VOLTAGE OF 249 VOLTS CAN BE ACHIEVED. MAXIMUM THRESHOLD
*		VOLTAGE MAY HAVE TO CHANGE AS WELL-MUST BE TESTED. SEE
*		TMPVOLT1.TXT FILE FOR TEMP VS CURRENT AND VOLTAGE BASED ON
*		EAGLE-PITCHER 3rd ORDER EQUATION.
*
* 8 JUL 93 - TOTAL AMP-HOURS VERIFIED FOR THE FOLLOWING CONFIGURATIONS:
*		SYSTEM	CHARGE CURRENT	TARGET A*HR	MEASURED
* OLD SYSTEM	3-PHASE     40 AMPS	 270		208 VERSION D SOFTWR
* NEW SOFTWARE, VERSION E
* 6 JUL 93	3-PHASE     40 AMPS	 270		275 (AT 228 VOLTS)
* 7 JUL 93	3-PHASE     20 AMPS	 270		275 (AT 244 VOLTS)
*		1-PHASE     20 AMPS	 270
*
* 8 JUL 93 - 100 A*HR MINIMUM REQUIRED FOR WATERING MUST BE ACCOMPANIED BY
*		A TOTAL A*HR COUNTER TO INSURE THAT BATTERIES ARE WATERED
*		EVERY 270 A*HRS.
*
* 10 JUL 93 - ADDED TOTAL AMP*HOUR COUNTER AFTER MAX THRESHOLD FOR CHARGE
*		CURRENT CYCLING IS REACHED (OR A SIMPLE THRESHOLD VOLTAGE FOR
*		SINGLE PHASE) BASED ON EAGLE PITCHER DATA, THE BATTERY WILL
*		CHARGE FOR X(TO BE DETERMINED) AMP*HRS ONCE THIS THRESHOLD
*		IS REACHED.  THIS PREVENTS OVERCHARGING A BATTERY THAT IS
*		NOT FULLY DISCHARGED AND CANNOT REACH MAX THRESHOLD VOLTAGE.
*
* 11 AUG 93 - CREATED NEW CHARGE PROFILE BASED ON TEST DATA AND CONVERSATION
*	WITH DON WHITMAN OF EAGLE PITCHER.  THE NEW CHARGE PROFILE IS:
*	A) CHARGE TO A THRESHOLD VOLTAGE BASED ON TEMPERATURE AT 40 AMPS
*	B) IF THE THRESHOLD IS NEVER REACHED IT WILL STOP CHARGING AT 270 AHR
*	C) AFTER THRESHOLD IS REACHED, A LOWER CURRENT WILL BE USED BASED ON
*	   TEMPERATURE.
*	D) REMAINDER OF CHARGE IS BASED TOTAL ACCUMULATED AHRS DURING THE
*	   FIRST PART OF THE CHARGE. USING DATA FROM EAGLE PITCHER, REMAINING
*	   AHRS ARE RETRIEVED FROM A LOOK-UP TABLE.
*
* 12 AUG 93 - ADDED MAINTAINANCE CHARGE. THIS INCLUDES THE FOLLOWING:
*	A) A TOP OFF CHARGE WILL OCCUR AT 30 MINUTE INTERVALS, THAT IS THE
*	   CHARGE WILL RESTART 30 MINUTES AFTER THE LAST TOP OFF CHARGE
*	   THIS CHARGE WILL BE BASED ON THE SAME TEMP VS VOLTAGE PROFILE USED
*	   ON A REGULAR CHARGE WITH A 10 A*HR MAXIMUM PER CHARGE.  IF OVER
*	   THE COURSE OF 48 HOURS THE TOTAL AMP*HOUR INPUT IS OVER 270 A*HRS
*	   ADDITIONAL WATERING MAY BE REQUIRED ON THE NEXT CHARGE CYCLE EVEN
*	   IT ONLY LASTS < 100 A*HR? MUST BE INVESTIGATED.
*
* 20 AUG 93 - ADDED BATTERY THERMAL CHARGE DELAY AND CHANGED THERMAL LIMITS
*	CHARGE DELAY IS BASED ON TIME VERSUS TEMPERATURE. EACH TEMPERATURE
*	HAS A MAXIMUM WAIT PERIOD ASSOCIATED WITH IT. 45 DEGREES AND ABOVE
*	IS 4 HOURS. MAXIMUM START CHARGE TEMPERATURE IS 45 DEGREES. THE
*	TEMPERATURE CAN RISE TO 50 DEGREES DURING CHARGING, HOWEVER IT
*	CANNOT EXCEED THIS TEMP.  35 DEGREES IS THE CUT-OFF POINT FOR THERMAL
*	DELAY.	A LINEAR RELATIONSHIP BETWEEN 35 AND 45 DEGREES AND 0 TIME
*	TO 4 HOURS EXISTS.
*	NOTE: AC MUST BE PRESENT FOR THIS ROUTINE TO WORK.  IF AC IS REMOVED
*	FOR MORE THAN 2 MINUTES, THE PROGRAM WILL START FROM FULL INITIALIZ.
*
******
*
*
*RAM	 MAP	 $0050-$00BF
*STACK	 MAP	 $00C0-$00FF
*INTERUPT VECTORS TABLE
*	$1FF4-$1FF5 - OVERFLOW
*	$1FF6-$1FF7 - TIMER OUTPUT COMPARE
*	$1FF8-$1FF8 - TIMER INPUT CAPTURE
*	$1FFA-$1FFB - EXTERNAL INTERRUPT
*	$1FFC-$1FFD - SOFTWARE INTERRUPT
*PROGRAM COUNTER
*	$1FFE-$1FFF - INITIAL VALUE
*TIMER CLOCK - 2usec
*INTERNAL BUS CLOCK - 2MHz
*OSC.CLOCK - 4MHz


		ORG	$0		;SET ORGIN TO 0
PORTA		EQU	$0		;DEFINE RAM MEMORY MAP
PORTB		EQU	$1
PORTC		EQU	$2
PORTD		EQU	$3
PA_DIR		EQU	$4		;PORTA DIR REG
PB_DIR		EQU	$5		;PORTB DIR REG
PC_DIR		EQU	$6		;PORTC DIR REG
EPRC		EQU	$7		;EPROM CONTROL
AD_RR		EQU	$8		;AD RESULT REG
AD_CR		EQU	$9		;AD CONTROL REG
PLMA		EQU	$0A		;PULSE LENGTH MODULATION A
PLMB		EQU	$0B		;PULSE LENGTH MODULATION B
MISCR		EQU	$0C		;MISC REG
BRATE		EQU	$0D		;SCI BAUD RATE REG
SCCR1		EQU	$0E		;SCI CONT REG1
SCCR2		EQU	$0F		;SC1 CONT REG2
SCSR		EQU	$10		;SCI STATUS
SCDAT		EQU	$11		;SCI DATA
TCR		EQU	$12		;TIMER CONTROL
TSR		EQU	$13		;TIMER STATUS
CAPH1		EQU	$14		;CAP HI 1
CAPL1		EQU	$15		;CAP LO 1
CMPH1		EQU	$16		;COMP HI 1
CMPL1		EQU	$17		;COMP LO 1
CTH		EQU	$18		;COUNTER HI
CTL		EQU	$19		;COUNTER LO
ACTH		EQU	$1A		;ALT COUNTER HI
ACTL		EQU	$1B		;ALT COUNTER LO
CAPH2		EQU	$1C		;CAP HI 2
CAPL2		EQU	$1D		;CAP LO 2
CMPH2		EQU	$1E		;COMP HI 2
CMPL2		EQU	$1F		;COMP LO 2


		ORG	$50		;START RAM VARIABLES

TFlag		EQU	$50
Timer1		EQU	$51
Timer2		EQU	$52
Timer3		EQU	$53
Timer4		EQU	$54
Timer5		EQU	$55
Timer6		EQU	$56
Timer_sec	EQU	$58
Timer_Save	EQU	$59
ACC_AD		EQU	$5A
*

Timer_status	EQU	$61
BIT_passed_cnt	EQU	$62
BIT_cnt		EQU	$63
Soft_start_sc	EQU	$64
Soft_start_cnt	EQU	$65
GFCI_emergency	EQU	$66
One_second_cnt	EQU	$67		;bit mode timer
*
** MAINTAINANCE CHARGE EQUATES
*
*
MAINT_FLG	EQU	$69	;
MAINT		EQU	$00	; INDICATES A MAINTAINANCE CHARGE
STP_MAINT	EQU	$01	; INDICATES END OF MAINTAINANCE
MAINT_CYC	EQU	$02	; INDICATES 1 HOUR HAS ELASPED WHEN SET
CARRY		EQU	$03	; USED TO KEEP TRACK HOURS
*
STP_AHR_CTR	EQU	$6A	;
MAINT_AHR_TTL	EQU	$6B	; TOTAL MAINTAINANCE AMP HOURS
MAINT_AHR_CTR	EQU	$6C	; KEEPS TRACK OF ACCUMULATED A*HRS DURING MAI
MAINT_TMR	EQU	$6D	; KEEPS TRACK OF MINUTES DURING MAINT CHARGE
MAINT_HRS	EQU	$6E	; KEEPS TRACK OF TOTAL HOURS DURING MAINT
*
** NEW EQUATES - ADDED 8 JUN 93
*
AHR_TOTL_LO	EQU	$72	;
AHR_TOTL_HI	EQU	$73	; STORE TOTAL AMP*HOURS SINCE LAST WATER
*
VBATT_TEMP_MAX	EQU	$74	; MAXIMUM BATTERY VOLTAGE BASED ON TEMP
B_TEMP_PTR	EQU	$75	; BATTERY TEMPERATURE PROFILE TABLE POINTER
BATT_TEMP_CMP	EQU	$76	; TMP REG FOR COMPARISON OF BATTERY TEMP
CURRENT_LO	EQU	$78	; FINAL CHARGE (CUTBACK) CURRENT
CURRENT_HI	EQU	$C8	; 3 PHASE HIGH CURRENT CHARGE (40 AMPS)
*		NOTE	THIS IS A CONSTANT, NOT VARIABLE
THRHLD_CTR	EQU	$79	;
ILOW_TOTL	EQU	$7A	; 1/4 TOTAL 40 A*HR CHARGE
IHIGH_END	EQU	$7B		;
**
* MORE FLAGS
NIFE_FLG	EQU	$7C	;
LOW_END		EQU	$00	; NICKEL IRON FLAG
AC_PRS_CHG	EQU	$01	; CONTINUE MAINTAINANCE CHARGE ONLY IF AC IS
*				; PRESENT
NIFETMP		EQU	$7F	; TEMP REGISTER
*
*
IFlag		EQU	$80
Ibatt		EQU	$81
Vbatt		EQU	$82
Icmd		EQU	$83	;CHARGE CURRENT - LOADED INTO PLMA
Z		EQU	$84
V_CYCLE EQU	$85

Vh		EQU	$87
Vl		EQU	$88
Th		EQU	$89
Tl		EQU	$8A
ACl		EQU	$8B
ACh		EQU	$8C

Batt_temp	EQU	$8D
Pwr_src		EQU	$8E
Pwr_src_cnt	EQU	$8F
AC_present	EQU	$90
GWIT_In		EQU	$91

Itotal		EQU	$92
Itotal_hi	EQU	$93
Itotal_mi	EQU	$94
Itotal_lo	EQU	$95
GWIT_cnt	EQU	$96
Pump_wait_cnt	EQU	$97
Purge_wait_cnt	EQU	$98
Delay_cnt	EQU	$99
Delay_cntr	EQU	$9A
Batt_temp_thr	EQU	$9B
*
BATT_WAIT_FLG	EQU	$9C	; FLAGS USED WHILE WAITING FOR COOL BATTERY
CHECK_BAT_TEMP	EQU	$00	; CLEARED ONLY ON FULL INITIALIZATION
AC_PRS_WT	EQU	$01	; AC PRESENT DURING CHARGE DELAY FLAG
CHK_4SEC	EQU	$02	; CHECK BATT TEMP FOR 4 SECONDS
BATT_COOL_TIME	EQU	$9D	; TOTAL TIME BATTERY HAS BEEN COOLING
batt_cool_cklo	EQU	$9E	; THESE REGISTERS ALLOW ACCURATE TEMP
batt_cool_ckhi	EQU	$9F	; CHECK OF THE BATTERIES (SAMPLE 4-6 SEC)
*
BATT_VOLT_MAX	EQU	$A0
BATT_MAX2	EQU	$A1
*
CHGR_TEMP_VAL	EQU	$B0
CHGR_TEMP_HI	EQU	$B1
CHGR_TEMP_MAX	EQU	$B2
CL		EQU	$B3	;MACRO FILTER VARIABLE FOR CHARGER TEMPERATUR
CH		EQU	$B4
CHGR_TMP_FLG	EQU	$B5	;CHARGER TEMPERATURE FLAGS
*
TEMP1		EQU	$B8
TEMP2		EQU	$B9
AC_SYNC_CNT	EQU	$BA	;ADDED 11 MAY 93 -
AC_SYNC_TMP	EQU	$BB	;ADDED 11 MAY 93
*
*
***********	  PORTA     *************************************
*	  O	CHARGE	Out	Old->	STARTSW     I	 1=CLOSED
*	  1    -	Out		PARK\N	    I	 1=CLOSED
*	  2    -	Out		DRIVE SW    I	 1=CLOSED
*	  3    -	Out		CLUTCH SW   I	 1=CLOSED
*	  4    -	Out		DATA IN     O	 EE PROM
*	  5    -	Inp		STORE HD    I
*	  6    -	Out		DISP/SK     O
*	  7	WDT	Out		WDT	    O
***********	  PORTB     *************************************
*	  0	MUXA	Out	Old->	MUXA
*	  1	MUXB	Out		MUXB
*	  2	QWIT	Out		FD ENA		1=ON
*	  3	H2O PUM Out		1AD ENA		1=ON
*	  4	FFRST	Out				1=ON
*	  5	Pre_Q	Out				1=ON
*	  6    -	Out		PAR		1=RESET
*	  7	H2O PUR Out		RD ENA		1=ON
***********	  PORTC     *************************************
*	  0    -	Out	Old->			0=ON
*	  1	MAIN_AC Out		LD		1=ON
*	  2    -	Out				1=PLUG
*	  3    -	Out				0=ON
*	  4    -	Out		EE PROM CS
*	  5    -	Out		D OSC %		MAXI
*	  6    -	Out		0=ON
*	  7    -EE SK	Out		EE SK
***********   ANALOG INPUTS  ************************************
IBatt_Avg_mux	EQU	%00		;an.input Filtered Output current
IBatt_Avg_bit	EQU	6		;AN(6) ANALOG PORT
Vbatt_Sens_mux	EQU	%01		;an.input sensed battery voltage
Vbatt_Sens_bit	EQU	1
Vin_mux		EQU	%01		;an.input sensed Rectified voltage
Vin_bit		EQU	0
Batt_Temp_mux	EQU	%00		;an.input Battery temperature
Batt_Temp_bit	EQU	1
Pwr_Src_mux	EQU	%11		;an.input indicator of 3 AC sources
Pwr_Src_bit	EQU	0
AC_Present_mux	EQU	%00		;an.input Sensed Rectified Voltage
AC_Present_bit	EQU	0
GWIT_In_mux	EQU	%10		;an.input Voltage from Green Wire test
GWIT_In_bit	EQU	0
***********   DIGITAL I/O    ************************************
GWIT_Out_port	EQU	PORTB		;output Current pulse on Gr.Wr. check
GWIT_Out_bit	EQU	5
FFRST_port	EQU	PORTB		;output FlipFlopReset
FFRST_bit	EQU	4
Pre_Q_Coil_port EQU	PORTB		;output Coil volt. for Precharge relay
Pre_Q_Coil_bit	EQU	2
H2O_Pump_port	EQU	PORTB		;output to run pump motor
H2O_Pump_bit	EQU	3
H2O_Purge_port	EQU	PORTB		;output to run pump to purge
H2O_Purge_bit	EQU	7
Main_AC_On_port EQU	PORTC		;output Coil volt. for Main AC contact
Main_AC_On_bit	EQU	1
Charge_port	EQU	PORTA		;light on if main frame is running
Charge_bit	EQU	0
*****************************************************************


*Icmd CALCULATION CONSTANTS
A1		EQU	$80
A2		EQU	$80

*IFLAG SETUP
Vbatt_max	EQU	$00		;flag bit #0: Vbatt >= 249 volt
Charge_comp	EQU	$01		;flag bit #1: battery charge > 270AmpHr
Over_temp_ss	EQU	$02		;over temperature soft start flag
SINGLE_PHASE	EQU	$03	;SINGLE PHASE POWER SOURCE
BATT_HOT	EQU	$04	;
CHGR_CUTBK	EQU	$05	;
CHGR_SHUTDWN	EQU	$06	;
LOW_I_CHG	EQU	$07	;


*
ITOTAL_MAX	EQU	$D8		;maximum of total charge - 270AmpHr
					;1.25 A*HR PER COUNT
Batt_temp_lo	EQU	$44		;high temperature hysteresis
Batt_temp_hi	EQU	$46		;low temperature hysteresis
BATT_TEMP_MIN	EQU	$C3	;MINIMUM TEMPERATURE FOR CURRENT VS TEMP
*			PROFILE (TEMPS BELOW THIS WILL NOT AFFECT PROFILE)
BATT_TEMP_MAX	EQU	$44	;MAXIMUM TEMPERATURE FOR CURRENT VS TEMP
*			PROFILE (TEMPS ABOVE THIS WILL NOT AFFECT PROFILE)
SINGLE_PHS_MAX	EQU	$64	; 20 AMPS MAXIMUM ON SINGLE PHASE
*
*
I_LO_DEFAULT	EQU	$64	;LOW CUTBACK CURRENT DEFAULT VALUE
V_HI_DEFAULT	EQU	$7D	;HIGH VOLTAGE CHARGE FINISHED DEFAULT VALUE
*
*CHARGER THERMAL
*
CHGR_HYST_LO	EQU	$24		;CHARGER ALREADY HOT CUTBACK
CHGR_HYST_HI	EQU	$4E		;NORMAL CUTBACK POINT
CHGR_MAX	EQU	$65		;MAXIMUM CHARGER TEMP
*
*TIMER CONTROL REGISTER
ICIE		EQU	$7		;Input Capture Interrupt Enable
OCIE		EQU	$6		;Output Compare Interrupt Enable
TOIE		EQU	$5		;Timer Overflow Interrupt Enable


*TIMER STATUS REGISTER
ICF1		EQU	$7		;Input Capture Flag 1
OCF1		EQU	$6		;Output Compare Flag 1
TOF		EQU	$5		;Timer Overflow Flag
ICF2		EQU	$4		;Input Capture Flag 2
OCF2		EQU	$3		;Output Compare Flag 2

*MAINTANANCE CONSTANTS
STP_MAINT_AHR	EQU	$14	;20*1.25 A*HR = 25 A*HRS PER MAINT CYCLE
MAX_MAIN_RPT	EQU	$03	; NO MORE THAN 3, 25 A*HR CYCLES
*
*SOFTWARE TIMER CONSTANTS
Timer1_flag	EQU	$7
Timer1sec	EQU	Timer1_flag
Timer1_const	EQU	!1		;1 sec
Timer2_flag	EQU	$6
Timer2sec	EQU	Timer2_flag
Timer2_const	EQU	!2		;2 sec
Timer3_flag	EQU	$5
Timer10sec	EQU	Timer3_flag
Timer3_const	EQU	!10		;10 sec
Timer4_flag	EQU	$4
Timer30sec	EQU	Timer4_flag
Timer4_const	EQU	!30		;30 sec
Timer5_flag	EQU	$3
Timer1min	EQU	Timer5_flag
Timer5_const	EQU	!60		;1 min
Timer6_flag	EQU	$2
Timer2min	EQU	Timer6_flag
Timer6_const	EQU	!120		;2 min
TIMER_10MS_FLAG EQU	$0
T_ms_update_lo	EQU	$88		;10ms
T_ms_update_hi	EQU	$13
Timer_sec_flag	EQU	$1
T_sec_update_lo EQU	$50
T_sec_update_hi EQU	$C3

***********	  MACROS    *************************************
*FLT OUT LSB		%1
*FLT OUT MSB		%2
$MACRO	FILTER
		LDA	%1
		SUB	%2		      ;Vbatt*(1-1/256)
		STA	%1
		LDA	%2
		SBC	#$00
		STA	%2
		BHS	LL
		CLR	%2
		CLR	%1
LL:
		LDA	%1
		ADD	AD_RR
		STA	%1
		LDA	%2

		ADC	#$00
		STA	%2
$MACROEND

*AD_CR CODE = 20h+Bit#	---->	%1
$MACRO	AtoD
		LDA	#$%1
		STA	AD_CR
		LDA	#$40
AtoD_A:
		DECA
		BNE	AtoD_A
		BRCLR	7,AD_CR,*		;loop until A/D completed
$MACROEND

		ORG	$0100
OPTR		DB	0


		ORG	$0800		;START EPR
INITO:
*
*09 SEP 93 - WHEN THE BATTERY IS UNPLUGGED, THE PROCESSOR RESTARTS AT THIS
*	ADDRESS AND FORCES WATERING IF THE BATTERY IS CHARGED FOR A MINIMUM
*	OF 50 AMP*HRS.
*
	LDA	#$B0	; INITIALIZE TO 220 A*HR ON BATTERY POWER-UP
	STA	AHR_TOTL_LO	; STORE TOTAL AMP*HOURS SINCE LAST WATER
	CLR	AHR_TOTL_HI	; CLEAR MSB OF TOTAL A*HRS
*
***********	 PROGRAM    *************************************
*INITIALIZATION

INIT0:
		CLR	BATT_WAIT_FLG	;
ZERO_INIT:

*DISABLE INTERRUPT
		SEI			;set interrupt mask, disa interrupt
*RESET STACK POINTER
		RSP
*RESET PORT DIRECTION
		LDA	#$00		;input mode and write into latch
		STA	PA_DIR
		STA	PB_DIR
		STA	PC_DIR
*INITALIZE PORT OUTPUT
		LDA	#$00
		STA	PORTA
		LDA	#$80
		STA	PORTB
		LDA	#$00
		STA	PORTC
*SET PORT DIRECTION BITS
		LDA	#$DF		;1=output, 0=input
		STA	PA_DIR		;all inputs
		LDA	#$0FF		;1=output, 0=input
		STA	PB_DIR		;all outputs
		LDA	#$0FF		;1=output, 0=input
		STA	PC_DIR		;all outputs
*RESET PULSE LENGTH MODULATORS
		LDA	#$00
		STA	PLMA		;reset PLMA
		LDA	#$80
		STA	PLMB		;reset PLMB
*SET A/D STATUS/CONTROL REGISTER:
		LDA	#$20
		STA	AD_CR		;enable A/D
*ENABLE GROUND FAULT INTERRUPT AND
*SET MISCELLANEOUS REGISTER:		;disable WatchDog
		LDA	#$D0	;10	;negative edge,lowlevel sens.interrupt
		STA	MISCR		;enable ext.interrupt
					;fast mode of PLM, System clock
*SET OPTIONAL REGISTER
		LDA	#$08
		STA	OPTR		;EEPROM read only, security is active
*ECLK REGISTER
		LDA	#$00	;
		STA	EPRC		;

*RESET SERIAL COMMUNICATION REGISTER
		CLR	SCCR2
*RESET MAINTAINANCE FLAGS
		CLR	MAINT_FLG	; ONLY CLEARED ON FULL INIT
		CLR	STP_AHR_CTR	; COUNTER FOR MAX AHR SHUTDOWN CYCLES
		CLR	MAINT_AHR_TTL	;TOTAL NUMBER OF AMP*HRS DURING MAINT
		CLR	MAINT_AHR_CTR	;ACCUMULATED A*HRS DURING MAINT
		CLR	MAINT_TMR	;MINUTES DURING MAINT CHARGE
		CLR	MAINT_HRS	;TOTAL HOURS DURING MAINT
*
** MAINTANCE INITIALIZATION STARTS HERE, FULL INITIALIZATION OCCURS ABOVE.
*
MAINT_INIT:
		CLR	IFLAG		;clear Input data flags
		CLR	BATT_VOLT_MAX	;CLEAR MAXIMUM VOLTS FLAG
		CLR	NIFE_FLG	; CLR LOW_END

*RESET TIMER REGISTERS
		CLR	TCR		;interrupts are disable,negative edge,
					;output level is low.
*CLEAR COMPARE TIMER INTERRUPTS
		JSR	CMP_intrpt_clear	; THIS IS DONE ON INIT
*RESET S/W TIMER REGISTERS
		CLR	TFlag		;clear Timer flags
		CLR	Timer1
		CLR	Timer2
		CLR	Timer3
		CLR	Timer4
		CLR	Timer5
		CLR	Timer6
		CLR	One_second_cnt
*
*RESET TOTAL BATTERY CHARGE (AmpHrs) VARIABLES
		CLR	Itotal
		CLR	Itotal_hi
		CLR	Itotal_mi
		CLR	Itotal_lo
*RESET FILTER VARIABLES
		LDA	#$80
		STA	Vh	;BATTERY VOLTAGE MACRO VARIABLE
		CLR	Vl	;
		LDA	#$4F	; INITIALIZE BATTERY TEMP
		STA	Th	;BATTERY TEMPERATURE MACRO VARIABLE
		CLR	Tl	;
		CLR	Ch	;CHARGER TEMPERATURE MACRO VARIABLE
		CLR	Cl	;
		LDA	#Batt_temp_lo	;THRESHOLD FOR CHARGE CUTBACK
		STA	Batt_temp_thr
*CLEAR Icmd
		CLR	Icmd
*CLEAR COUNTERS
		CLR	Pump_wait_cnt
		CLR	Purge_wait_cnt
*ENABLE INTERRUPT
		CLI			;reset interrupt mask, ena interrupt
*INITIALIZE AC PRESENT FILTER DATA
		LDA	#$F0
		STA	ACh
		STA	ACl
*
**** INITIALIZE CHARGER TEMPERATURE FLAGS
*
*CHGR_TEMP
	LDA	#CHGR_HYST_HI	;INITIALIZE CUT-BACK TEMPERATURE
	STA	CHGR_TEMP_HI	;
	LDA	#CHGR_MAX	;INITIALIZE SHUTDOWN TEMPERATURE
	STA	CHGR_TEMP_MAX	;
	CLR	CL		;MACRO FILTER VARIABLE FOR CHARGER TEMP
	CLR	CH		;TEMPERATURE COOL, RESISTANCE LOW, VOLTAGE
*				;LOW
	CLR	CHGR_TMP_FLG	;CHARGER TEMPERATURE FLAGS
*
**** INITIALIZE BATTERY TEMPERATURE REGISTER
*
	LDA	#$80		; INITIALIZE BATTERY TEMPERATURE TO
	STA	BATT_TEMP	; 25 DEGREES CELCIUS
	CLR	BATT_COOL_TIME	; INITIALIZE COOLING TIMER
*
* INITIALIZE CHARGE PROFILE
*
	LDA	#I_LO_DEFAULT	; DEFAULT FINAL CHARGE CURRENT
	STA	CURRENT_LO	; FINAL CHARGE CURRENT INITIALIZED TO 20 AMPS
	LDA	#V_HI_DEFAULT	; INITIALIZE MAXIMUM BATTERY VOLTAGE TO
	STA	VBATT_TEMP_MAX	; 249 VOLTS
*
*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*START-UP CONTROL
*AC_PRESENT TEST
*LOOP AC_present_test
AC_present_test:
*READ AC_present
	JSR	AC_read
*UNTIL AC_present is TRUE (bit0=1)
	BRCLR	0,AC_present,AC_present_test
*FLIP FLOPS RESET
	BSET	FFRST_bit,FFRST_port
*GREEN WIRE TEST
GWITest:
*SYNCRONIZATION OF GWIT.
*Delay Synch AND READING OF INPUTS SIGNALS
	JSR	Delay_synch
*OUTPUT GWIT_OUT					;5,PORTB
	BSET	GWIT_Out_bit,GWIT_Out_port	;output GWIT_Out
*				; PULL IN K3 RELAY LOOK FOR GW
*Delay 500ms
		JSR	Delay_500ms
*READ GWIT_IN
		JSR	GWIT_In_read
*RESET GWIT
		BCLR	GWIT_Out_bit,GWIT_Out_port	;output GWIT_Out
*IF GWIT_IN IS FAILED  (bit0=1)
		BRSET	0,GWIT_In,Bit_OK
*THEN TEST AC_present
		JSR	AC_read
*IF AC_present IS FAILED THEN GOTO AC_present_test
		BRCLR	0,AC_present,AC_present_test
*REPEAT GWITest UNTIL GWITest IS PASSED
		JMP	GWITest
*
**** NEW SECTION TO CHECK BATTERY TEMPERATURE AND DELAY CHARGE IF IT IS TOO
*    HOT TO BEGIN CHARGING.
*
BIT_ok:
	BRCLR	CHECK_BAT_TEMP,BATT_WAIT_FLG,CHECK_BATT ;
	JMP	BATT_TEMP_OK	; BRANCH IF BATT TEMP HAS COOLED
*
** THIS SECTION READS THE BATTERY TEMPERATURE SEVERAL TIMES AND DELAYS
*  CHARGING IF IT IS HOT. THE TEMPERATURE IS CHECKED IN TWO MINUTE INTERVALS.
*  THE CHARGING LIGHT AND FANS CAN REMAIN ON UP TO FOUR HOURS. AFTER THIS THE
*  FANS ARE TURNED OFF AND THE BATTERY MUST COOL TO AT LEAST 45 DEGREES C
*  BEFORE CHARGING CAN START.
*
CHECK_BATT:
NIFE_BATT_DEL:
NIFE_BATT_DEL1:
*	NOTE: SHOULD PROBABLY CHECK THIS FOR 1-2 SECONDS BEFORE CONTINUING TO
*	INSURE A TRUE BATTERY TEMPERATURE HAS BEEN READ.
*
	BCLR	6,TCR		;
	CLR	batt_cool_cklo	;
	CLR	batt_cool_ckhi
CONT_BATT_CHK:
	CLR	TEMP1
	JSR	BATT_TEMP_READ	;CHECK BATTERY TEMPERATURE
BATT_TEMP_RDST:
	BCLR	0,PORTB		;set MUX CH 00
	BCLR	1,PORTB		;
	AtoD	21		;
	FILTER	Tl,Th		;
	STA	BATT_TEMP	;
	INC	TEMP1		;
	LDA	TEMP1		;
	CMP	#$1F		;
	BHS	CONT_CHK1	;
	BRA	BATT_TEMP_RDST	;
CONT_CHK1:
	JSR	TIMER_MS_INIT	;
	LDA	TSR		; INITIALIZE 10 mSEC TIMER
*WAIT 6 ms DELAY
	JSR	Delay_6ms
**
* EACH LOOP 10 mSEC, CHECK BATT TEMP FOR 2 SECONDS BEFORE CONTINUING
*
	BSET	6,TCR			; ENABLE TIMER
	BSET	TIMER_10MS_FLAG,TFLAG	;
	BRSET	TIMER_10MS_FLAG,TFlag,* ; WAIT TILL 10 mSEC TIMED OUT
	BCLR	6,TCR			; DISABLE TIMER
*
*RESET CPM TIMERS INTERRUPT
	INC	batt_cool_cklo		;
	LDA	batt_cool_cklo		;
	CMP	$C8			; 200 TIMES THROUGH LOOP IS 2 SECONDS
	BHS	CONT_BATT3		;
	JMP	CONT_BATT_CHK		;
CONT_BATT3:
	INC	batt_cool_ckhi
	LDA	batt_cool_ckhi
	CMP	#$0F
	BHS	CONT_BATT2
	JMP	CONT_BATT_CHK
*
CONT_BATT2:
*
*	BATTERY TEMPERATURE WAS READ FOR 2 SECONDS IN PREVIOUS SECTION
*	CHECK BATTERY TEMPERATURE VERSUS TIME IN THIS SECTION
*
	LDA	BATT_TEMP	; GET BATTERY TEMPERATURE
	CMP	#$4B		; SEE IF IT IS AT MAX TEMP
	BHS	CONT_DLY_CHK	; LOAD MAXIMUM DELAY TIME (4 HOURS)
	CLRX
	BRA	GET_DELAY_TM	;
CONT_DLY_CHK:
	LDA	BATT_TEMP	;
	SUB	#$4B		; TABLE STARTS AT ADDRESS 0, SUBTRACT OFFSET
	STA	TEMP2		;
	CMP	#$18		; COMPARE TO 35 DEGREE CUTOFF DELAY TEMP
	BLO	CONT_DLY_CHK2	; ie TEMPS BELOW THIS WILL NOT CAUSE DELAY
	CLR	TEMP2		;
	BRA	GET_DELAY_TM1	;
CONT_DLY_CHK2:
	LDX	TEMP2		;
GET_DELAY_TM:
	LDA	CHARGE_DELAY,X	; GO TO LOOK-UP TABLE, LOAD
	STA	TEMP2		; CALCULATED DELAY TIME
GET_DELAY_TM1:
	LDA	TEMP2		;
	CMP	BATT_COOL_TIME	; COMPARE TO ACTUAL COOL TIME
	BLS	BATT_COOL_CNT	; START CHARGE IF COOLING TIME IS GREATER
* THAN VALUE IN TABLE
HIGH_TMP:
	LDA	BATT_COOL_TIME	;
	CMP	#$78		; SEE IF BATTERY HAS BEEN COOLING > 4 HOURS
	BLO	BATT_WAIT_COOL	;
*	BSET	FANS_OFF,BATT_WAIT_FLG	; CONTINUE WAIT, TURN OFF FANS
	BCLR	0,PORTA		; TURN OFF FANS, BATT NOT COOL AFTER 4 HRS
	BRA	BATT_CNTD	; SKIP NEXT SECTION
BATT_WAIT_COOL:
	INC	BATT_COOL_TIME	; EACH INCREMENT IS APPROXIMATELY 2 MINUTES
	BSET	0,PORTA		; MAKE SURE CHARGE LIGHT AND FANS ARE ON.
BATT_CNTD:
*INITIALIZE SEC. TIMERS
		JSR	Timer_sec_init
*ENABLE SEC TIMER INTERRUPT
		BSET	6,TCR		;enable  CMP interrupt
		BSET	Timer_sec_flag,TFlag
*SET 120SEC (2 MIN) FLAG
		BSET	Timer2min,Tflag
		BSET	AC_PRS_WT,BATT_WAIT_FLG ; AC PRESENT DURING CHARGE DELAY FLAG
*
AC_loss_120B:
*CHECK AC_present
	JSR	AC_read
*TEST THE TIMER120SEC
	BRSET	Timer2min,TFlag,AC_loss_less120B	; CONT 2 MIN
	BRSET	AC_PRS_WT,BATT_WAIT_FLG,LOSS_MORE_120B	;
	JMP	NIFE_BATT_DEL1	;
LOSS_MORE_120B:
	BRA	AC_loss_more120B
AC_LOSS_LESS120B:
*	BCLR	6,TCR		;disable  CMP interrupt
	BRCLR	0,AC_present,AC_LOSS_120B	; AC NOT PRESEN CONT
*AC PRESENT IF 0,ACPRESENT IS CLEAR
	BCLR	AC_PRS_WT,BATT_WAIT_FLG ; AC IS PRESENT, CLEAR FLAG
	BRA	AC_LOSS_120B		; CONTINUE 2 MIN DELAY
*
AC_LOSS_MORE120B:
	BCLR	6,TCR		;disable  CMP interrupt
	JMP	BATT_CHK_NOAC	; DO NOT CONTINUE, AC IS LOST
*
*
BATT_COOL_CNT:

	BCLR	6,TCR		;disable  CMP interrupt
	BSET	CHECK_BAT_TEMP,BATT_WAIT_FLG	; INDICATE COOL BATTERY
	JMP	ZERO_INIT	; RESTART PROGRAM, BUT DON'T CLEAR WAIT FLAG
*
BATT_CHK_NOAC:
	BCLR	CHECK_BAT_TEMP,BATT_WAIT_FLG	;
	JSR	AC_LOSS		;
	BRA	BATT_CHK_NOAC	; LOOP TILL AC IS PRESENT
*
*SET PRECHARGE RELAY
BATT_TEMP_OK:
		BSET	Pre_Q_Coil_bit,Pre_Q_Coil_port
*WAIT 5 SEC TO PRECHARGE RELAY
		JSR	Delay_5sec
*SET MAIN AC CONTACTOR
		BSET	Main_AC_On_bit,Main_AC_On_port
		BSET	5,PORTC		;BOOST COIL
		JSR	DELAY_6MS
		JSR	DELAY_6MS
		JSR	DELAY_6MS
		JSR	DELAY_6MS
		BCLR	5,PORTC
*SET SOFT START COUNTER
		LDA	#$00
		STA	Soft_start_sc
*INITIALIZE S/W TIMERS
		JSR	Timer_ms_init
		LDA	TSR		;Clear timer interrupts
		JSR	Timer_sec_init
		LDA	TSR		;Clear timer interrupts
*INPUT POWER SOURCE DISCRETE,
		JSR	Pwr_Src_read

*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*REAL-TIME POWER CONTROL
*LOOP MAIN FRAME
Main_frame:
*??????????????????
*
*NEW SECTION (8 JUL 93) MAKES LOOP TIME = 10 mSEC
		JSR	TIMER_MS_INIT	;
		LDA	TSR		; INITIALIZE 10 mSEC TIMER
		LDA	 Itotal_hi
		STA	 PLMB
		BSET	 0,PORTA
		JSR	 AC_read
*IF AC_PRESENT is FALSE (bit0=1)
*THEN CALL AC LOSS HANDLER ROUTINE
		BRSET	 0,AC_present,Main_frameA
*CALL AC LOSS ROUTINE
		JSR	AC_loss        ;if lost (bit0=0),call AC_loss
Main_frameA:

		BRCLR	 Charge_comp,IFlag,MAIN_VBATT	; 270 A*HR FLAG
		JMP	CHARGE_OVER	;
*
*** NOTE: ICMD IS ADJUSTED IN MORE THAN ONE LOOP.  THE TESTS BELOW ARE
* CONDUCTED BY PRIORITY. IF THE BATTERY VOLTAGE SETS THE CHARGE CURRENT TO
* ONE VALUE, IT CAN BE RESET BY THE BATTERY TEMPERATURE READ ROUTINE.
* CHARGER TEMPERATURE IS HIGHEST PRIORITY AFTER POWER SOURCE CORRECTION.
*
*READ Vbatt
MAIN_VBATT:
		JSR	Vbatt_read
*TEST IF Vbatt => 249 V
		BRCLR	LOW_END,NIFE_FLG,NOT_OVER	;
		JMP	CHARGE_OVER	;
NOT_OVER	BRCLR	Vbatt_max,IFlag,NOT_LOW_I	; FIRST VOLTAGE
*				; THRESHOLD
		BRCLR	MAINT,MAINT_FLG,NOT_OVER1	 ;
		JMP	CHARGE_OVER	; MAINTAINANCE CHARGE, DO NOT
NOT_OVER1	BRSET	LOW_I_CHG,IFLAG,NOT_LOW_I	; BRANCH IF INIT
		BSET	LOW_I_CHG,IFLAG
		LDA	ITOTAl		;
		STA	IHIGH_END	;
*
** BATTERY OVERCHARGE CALCULATION
*
* BATT TEMPORARY REGISTERS ARE USED HERE AND IN THE TEMPERATURE LOOP
*
* TOTAL A*HRS ARE IN 1.25 AHR INCREMENTS.  EACH OVERCHARGE A*HR IS BASED ON
* 10 A*HR INCREMENTS WHICH CORRESPONDS TO INCREMENTS OF 8 OR 8*1.24=10 A*HRS
*
	CLR	B_TEMP_PTR	; CLEAR POINTER FOR A*HR OVERCHARGE
	CLR	BATT_TEMP_CMP	; CLEAR TEMPORARY COMPARISON REGISTER
*
AHR_OVR_CHG:
	LDA	IHIGH_END	; LOAD TOTAL CHARGE AHRS
	CMP	BATT_TEMP_CMP	; COMPARE TO REFERENCE TO DETERMINE WHICH
	BLS	AHR_PROF	; VALUE IN TABLE TO USE
	INC	B_TEMP_PTR	; POINT TO NEXT VALUE IN TABLE
	LDA	BATT_TEMP_CMP	; LOAD REFERENCE
	ADD	#$08		; RESOLUTION OF AHR TABLE IS 8*1.25=10 AHRS
	STA	BATT_TEMP_CMP	;
	CMP	#$D8		;
	BHS	AHR_PROF	;
	BRA	AHR_OVR_CHG	; CONTINUE TO COMPARE
AHR_PROF:
	LDX	B_TEMP_PTR	; LOAD BATTERY TEMPERATURE POINTER
	LDA	AHR_FINISH,X	; GO TO LOOK-UP TABLE, LOAD
	STA	ILOW_TOTL	; STORE NEW MAXIMUM VOLTAGE
*
*COMPUTE Icmd USING Soft_start_sc
NOT_LOW_I	JSR	Icmd_compute
*READ Batt_temp AND ADJUST Icmd
	JSR	Batt_temp_read
	BRSET	BATT_HOT,IFLAG,ICMD_ZERO	; CURRENT = 0 IF BATTERY HOT
*
**READ CHGR_TEMP AND ADJUST ICMD ACCORDINGLY
*
	JSR	CHGR_TEMP	; READ CHARGER TEMP, CUT-BACK CURRENT IF
*				; NECESSARY
	BRSET	CHGR_SHUTDWN,IFLAG,ICMD_ZERO	; I = 0 IF CHARGER TO HOT
	BRSET	CHGR_CUTBK,IFLAG,PHASEHF	; CURRENT 1/2 IF HOT
*POWER SOURCE CORRECTION
		LDA	Pwr_Src
		BNE	Main_FrameB	;0,PWR_SRC IF TRUE
		BRA	PHASEHF		;
*
***** ADJUST CURRENT IF SINGLE PHASE SOURCE
*
MAIN_FRAMEB:
*
	BRCLR	SINGLE_PHASE,IFLAG,MAIN_FRAMEC	;BRANCH IF NOT SINGLE PHASE
*
PHASEHF LDA	Icmd
	CMP	CURRENT_LO	; REDUCE CURRENT BY 1/2 IF TOO HIGH FOR
	BHI	HALF_I		; SINGLE PHASE CHARGING
	LDA	CURRENT_LO	; GET LO CURRENT VALUE
	CMP	#SINGLE_PHS_MAX ; COMPARE TO MAXIMUM ALLOWABLE SINGLE PHASE
	BLO	MAIN_FRAMEC	; CURRENT
	LDA	#SINGLE_PHS_MAX ;
	STA	ICMD		; STORE MAXIMUM SINGLE PHASE CURRENT
	BRA	MAIN_FRAMEC	; SKIP NEXT SECTION
HALF_I	LDX	#$80	;			MSB	LSB
	MUL		;INDEX*ACC = 16 BIT	INDEX - ACC
	STX	Icmd	;STORE UPPER 8 BITS OF MULTIPLICAND IN ICMD
	BRA	MAIN_FRAMEC	;
*
* REDUCE CURRENT TO ZERO IF FAULT CONDITION EXITS
*

ICMD_ZERO	CLR	ICMD	; CURRENT = 0
*
Main_FrameC:
*
**** NEW SECTION ******
*NOTE: TOTAL AMP*HOUR SHUTDOWN ONLY WORKS IF THIS SUBROUTINE IS CALLED ONCE
*PER MAIN LOOP, WITH A LOOP TIME OF 10 mSEC
*Main_FrameB:
*
*CURRENT-HOURS CHARGE COMPUTATION - 30 APR 93 THIS ROUTINE HAS BEEN MODIFIED
* TO ESTIMATE TOTAL AMP HOURS - IT FORCES THE PROGRAM TO COMPLETE THE CHARGE
* AND GO TO THE WATERING SUBROUTINE IF 270 AMP*HRS HAVE ACCUMULATED AND THE
* BATTERY VOLTAGE IS NOT UP TO 249.  A COUNTER KEEPS TRACK OF THE TOTAL TIME
* IN HIGH CURRENT CHARGE MODE AND LOW CURRENT CHARGE MODE.  WHEN THE MSB OF
* THE COUNTER REACHES A PRE-DETERMINED VALUE, THE CHARGE_COMP FLAG IS SET.
*
**** NEW SECTION ******
* 10 MAY 93 - MODIFIED TO KEEP BETTER TIME
* 14 JUN 93 - MODIFIED TO STORE ACCURATE AMP*HOURS FOR SINGLE PHASE
*
	LDA	ICMD
	STA	IBATT	;
	JSR	 Itotal_comp
	LDA	Icmd
*OUTPUT Icmd TO POWER CONTROL H/W
	STA	PLMA
*WAIT 6 ms DELAY
		JSR	Delay_6ms
**
*NEW SECTION MAKES LOOP EXACTLY 10 mSEC
*
	BSET	6,TCR			; ENABLE TIMER
	BSET	TIMER_10MS_FLAG,TFLAG	;
	BRSET	TIMER_10MS_FLAG,TFlag,* ; WAIT TILL 10 mSEC TIMED OUT
	BCLR	6,TCR			; DISABLE TIMER
*
*RESET CPM TIMERS INTERRUPT
		JMP	Main_frame
*LOOP END MAIN FRAME

*!!!!!!!!!!!!!!!!!!!!!!! SHUTDOWN PROCEDURES !!!!!!!!!!!!!!!!!!!!!!!!!!

*========================================================================*
*SHUTDOWN PROCEDURES
Drop_all:
*RESET Icmd and ICMD-OUTPUT TO ZERO
		CLR	Icmd
		CLR	PLMA
*DROP MAIN CONTACTOR: RESET MAIN AC CONTACTOR
		BCLR	Main_AC_On_bit,Main_AC_On_port
*DROP PRECHARGE RELAY: RESET PRECHARGE RELAY
		BCLR	Pre_Q_Coil_bit,Pre_Q_Coil_port
*INITIALIZE SEC. TIMERS
		JSR	Timer_sec_init
		LDA	TSR		;Clear timer interrupts
		RTS

*========================================================================*
Start_precharge:
*WAIT 4 SECONDS
		BSET	Timer2sec,TFlag
		BRSET	Timer2sec,TFlag,*
		BSET	Timer2sec,TFlag
		BRSET	Timer2sec,TFlag,*
*SET MAIN AC CONTACTOR
		BSET	Main_AC_On_bit,Main_AC_On_port
		RTS

*========================================================================*
Charge_over:
*???????????????????
		lda   #$20
		sta   plmb
		BCLR	6,TCR	;DISABLE COMPARE INTERUPTS
*		BCLR	0,PORTA
*RESET Icmd TO 0
		CLR	Icmd
**
* UPDATE TOTAL AMP*HOUR REGISTER
*
	LDA	ITOTAL	; GET TOTAL AMP*HOUR COUNTER FOR THIS CHARGE
	ADD	AHR_TOTL_LO	; ADD TO LSB OF OVER ALL A*HR COUNTER FOR
	STA	AHR_TOTL_LO	; WATERING
	LDA	AHR_TOTL_HI	; GET MSB
	ADC	#$00		;
	STA	AHR_TOTL_HI	;
*
*STOP CHARGING
		LDA	#$00
		STA	PLMA
*CONTROL WATER PUMP
		JSR	Water_pump
*START FULL INITIALIZATION
		JMP	INIT0
		RTS

*========================================================================*
AC_loss:
*???????????????????
		lda	#$10
		sta	plmb
		BCLR	0,PORTA
		BCLR	6,TCR		;DISABLE COMPARE
*INITIALIZE SEC. TIMERS
		JSR	Timer_sec_init
*INITIALIZE MS TIMERS
		JSR	Timer_ms_init

*ENABLE SEC TIMER INTERRUPT
		BSET	6,TCR		;enable  CMP interrupt
		BSET	Timer_sec_flag,TFlag
*WAIT 2 SECONDS
		BSET	Timer2sec,TFlag
		BRSET	Timer2sec,TFlag,*
		JSR	AC_read
*IF AC_PRESENT is not TRUE (bit0=0)
		BRSET	0,AC_present,AC_loss_rts    ;if it's TRUE,no actions
*RESET Icmd
		CLR	Icmd
		CLR	PLMA
*DROP MAIN CONTACTOR: RESET MAIN AC CONTACTOR
		BCLR	Main_AC_On_bit,Main_AC_On_port
*WAIT	120 SECONDS
		BSET	Timer2min,Tflag
*LOOP AC LOSS DURING 120 SECONDS
AC_loss_120:
*CHECK AC_present
		JSR	AC_read
*TEST THE TIMER120SEC
		BRSET	Timer2min,TFlag,AC_loss_less120
		BRA	AC_loss_more120
AC_loss_less120:
*UNTIL AC_PRESENT is lost (bit0=0)
		BRCLR	0,AC_present,AC_loss_120
		LDA	#$00
		STA	Soft_start_sc
		JSR	Start_precharge
		BRA	AC_loss_rts
AC_loss_more120:
*STORE	Amp Hr. Counter
*DISABLE INTERRUPT
		BCLR	6,TCR		;disable  CMP interrupt
*START FULL INITIALIZATION
		JMP	INIT0
AC_loss_rts:
*DISABLE INTERRUPT
		BCLR	6,TCR		;disable  CMP interrupt
		RTS


*!!!!!!!!!!!!!!!!!!! WATER PUMP/PURGE ROUTINES !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

*========================================================================*
Water_pump:
*STOP CHARGING
		LDA	#$00
		STA	PLMA
		BCLR	Main_AC_On_bit,Main_AC_On_port ; DROP MAIN CTR
*
		BRCLR	MAINT,MAINT_FLG,WTR_CYC ; START WATERING IF NOT MAINT
		JMP	MAINTAINANCE	; CONTINUE MAINTAINENCE CYCLE
*** NEW SECTION **
*	THIS SECTION DETERMINES WHETHER THE BATTERY SHOULD BE WATERED
* BASED ON MIN ACCEPTABLE AMP*HOURS FOR THE CURRENT CHARGE TO ALLOW WATERING
* (100 A*HR PRELIMINARY).
*
WTR_CYC:
	LDA	ITOTAL	; GET TOTAL AMP HOURS FOR THIS CHARGE CYCLE
	CMP	#$50	; 1.25*80 AMP*HOURS MINIMUM REQUIRED
	BHS	WTR_START ; PASSES FIRST TEST, CONTINUE
*
** ADDED 8 JUN 93: THIS ENSURES BATTERIES ARE WATERED AFTER 270 A*HRS IF
*	SEVERAL SHORT CHARGES (LESS THAN 100 A*HRS) ARE ACCUMULATED.
*
WTR_CNT:		; THIS SECTION CHECKS FOR TOTAL A*HR SINCE LAST WATER
	LDA	AHR_TOTL_HI	;
	BNE	WTR_START	;
	LDA	AHR_TOTL_LO	; GET LOW BYTE
	CMP	#$D8	;
	BLO	NO_WTR	; DON'T WATER IF NOT ENOUGH A*HR
	BRA	WTR_START1	; START WATERING
NO_WTR	JMP	END_WTR		; OTHERWISE DON'T WATER
*
WTR_START1:
*
	LDA	ITOTAL	; GET TOTAL AMP HOURS FOR THIS CHARGE CYCLE
	CMP	#$28	; 1.25*40 = 50 AMP*HOURS MINIMUM REQUIRED IF A*HR -
	BHS	WTR_START ; TOTAL IS > 270
	BRA	NO_WTR	;
*START THE WATER PUMP
WTR_START	CLR	AHR_TOTL_LO	; CLEAR TOTAL AMP*HR REGISTER
		CLR	AHR_TOTL_HI	;
*
*START THE WATER PUMP
		BSET	H2O_pump_bit,H2O_pump_port
		JSR	Pump_wait	;11 MAY 93 - WAIT 26 MINUTES
*STOP THE WATER PUMP
		BCLR	H2O_pump_bit,H2O_pump_port
*START THE WATER PURGE
		BCLR	H2O_purge_bit,H2O_purge_port
		JSR	Purge_wait	;11 MAY 93 - WAIT 2 MINUTE
*STOP THE WATER PURGE
		BSET	H2O_purge_bit,H2O_purge_port
*START TRICKLE CHARGING
END_WTR		LDA	#$00
		STA	PLMA
		BCLR	0,PORTA		;TURN OFF CHARGE LIGHT
*WAIT UNTIL AC WILL BE LOST DURING 2 MINUTES
*
** START OF MAINTAINANCE CYCLE
*
MAINTAINANCE:
*
*** 30 MIN DELAY BEFORE MAINTAINANCE CHARGE BEGINS
*
*WAIT	120 SECONDS*15 = 30 MIN
* LOOP AC LOSS DURING 120 SECONDS: IF AC IS LOST FOR MORE THAN 2 MINUTES,
* ASSUME VEHICLE IS OPERATING AND DO NOT RESUME MAINTAINANCE CYCLE
*
*
** THIS SECTION OCCURS ONCE PER MAINT CYCLE
*
		BRCLR	STP_MAINT,MAINT_FLG,MAINT_CNT1	;CONT MAINTAINANCE
		JMP	MAINT_END	; END MAINTAINANCE
MAINT_CNT1:
		LDA	#$10		; LOAD 15
		STA	NIFETMP		; STORE IN COUNTER REGISTER
		BRSET	MAINT,MAINT_FLG,MAINT_CNT3	;
		BSET	MAINT,MAINT_FLG ; INDICATATE MAINTAINANCE CYCLE HAS
		BRA	NIFE_30MIN_DEL	; DON'T SAVE ITOTAL FIRST PASS
MAINT_CNT3	BRSET	MAINT_CYC,MAINT_FLG,HR_INC_MAINT
		BSET	MAINT_CYC,MAINT_FLG	; 30 MINUTES ELAPSED, INC
		BRA	MAINT_CNT2	;
HR_INC_MAINT	INC	MAINT_HRS	;
		BCLR	MAINT_CYC,MAINT_FLG	;
MAINT_CNT2:	LDA	MAINT_HRS	;
		CMP	#$30		;
		BLO	NIFE_MAINT_TTL	;
		JMP	MAINT_END	; 48 HOURS HAVE ELASPED, STOP MAINT
*
NIFE_MAINT_TTL:
		LDA	ITOTAL		; TOTAL A*HRS FOR LAST MAINT CYCLE
		ADD	MAINT_AHR_TTL	; ADD TO TOTAL MAINT A*HRS
		STA	MAINT_AHR_TTL	; SAVE NEW VALUE
		CMP	#$64		; 100*1.25= 125 A*HRS MAX
		BLO	NIFE_30MIN_DEL	;
		JMP	MAINT_END	;
*
*** THIS SECTION SECTION DELAYS CHARGING FOR 30 MINUTES.  AC PRESENT IS
*   CHECKED CONTINUOUSLY. IF LOST FOR UNDER 2 MINUTES, MAINT WILL CONTINUE.
*   IF AC IS LOST FOR MORE THAN 2 MINUTES, MAINT ENDS
*
NIFE_30MIN_DEL:
		BCLR	0,PORTA		;MAKE SURE CHARGE LIGHT IS OFF WHILE
*					;WAITING FOR MAINTAINANCE CHARGE
NIFE_30MIN_DEL1:
		JSR	Timer_sec_init	;INITIALIZE TIMERS
		BSET	6,TCR		;enable  CMP interrupt
		BSET	Timer2min,Tflag ;
		BSET	AC_PRS_CHG,NIFE_FLG ; AC PRESENT DURING WATERING FLAG
		DEC	NIFETMP		; COUNTER FOR 1/2 HOUR DELAY
		BEQ	MAINT_CNT	; START MAINT CHARGE, OTHERWISE CONT
AC_loss_120W:
*CHECK AC_present
		JSR	AC_read
*TEST THE TIMER120SEC
		BRSET	Timer2min,TFlag,AC_loss_less120W	; CONT 2 MIN
		BRCLR	AC_PRS_CHG,NIFE_FLG,NIFE_30MIN_DEL1	;
		BRA	AC_loss_more120W
AC_LOSS_LESS120W:
*		BCLR	6,TCR		;disable  CMP interrupt
		BRCLR	0,AC_present,AC_LOSS_120W	; AC NOT PRESEN CONT
*AC PRESENT IF 0,ACPRESENT IS CLEAR
		BCLR	AC_PRS_CHG,NIFE_FLG	; AC IS PRESENT, CLEAR FLAG
		BRA	AC_LOSS_120W		; CONTINUE 2 MIN DELAY
*
AC_LOSS_MORE120W:
		BCLR	6,TCR		;disable  CMP interrupt
		JMP	MAINT_END	; DO NOT CONTINUE, AC IS LOST
*
*
MAINT_CNT:

		BCLR	6,TCR		;disable  CMP interrupt
		JMP	MAINT_INIT	; JUMP TO MAINTAINANCE INTITIALIZATION
*
MAINT_END:
		JSR	AC_loss		;if it's more that 2min, than the
		JMP	MAINT_END	;AC_loss starts the full initialization
*
*========================================================================*
*11 MAY 93 - SHORTENED PUMP TIME TO 10 MINUTES
*18 MAY 93 - INCREASED PUMP TIME TO 26 MIN
Pump_wait:
		LDA	#$0D		;13*2 = 26 MIN
		STA	Pump_wait_cnt
Pump_waitA:
*INITIALIZE SEC. TIMERS
		JSR	Timer_sec_init
*ENABLE SEC TIMER INTERRUPT
		BSET	6,TCR		;enable  CMP interrupt
		BSET	Timer_sec_flag,TFlag
*SET 120SEC (2 MIN) FLAG
		BSET	Timer2min,Tflag
*WAIT	120 SECONDS
Pump_waitB:
*CHECK AC_present
		JSR	AC_read
		BRSET	0,AC_present,Pump_waitC
		JSR	AC_loss
		BSET	6,TCR		;restore 'enable  CMP interrupt'
Pump_waitC:
		BRSET	Timer2min,TFlag,Pump_waitB
		DEC	Pump_wait_cnt
		BNE	Pump_waitA
*RETURN
		RTS


*========================================================================*
*11 MAY 93 - SHORTENED PURGE TIME TO 2 MINUTES
*
Purge_wait:
		LDA	#$01	;1*2 = 2 MINUTES
		STA	Purge_wait_cnt
Purge_waitA:
*INITIALIZE SEC. TIMERS
		JSR	Timer_sec_init
*ENABLE SEC TIMER INTERRUPT
		BSET	6,TCR		;enable  CMP interrupt
		BSET	Timer_sec_flag,TFlag
*SET 120SEC (2 MIN) FLAG
		BSET	Timer2min,Tflag
*WAIT	120 SECONDS
Purge_waitB:
*CHECK AC_present
		JSR	AC_read
		BRSET	0,AC_present,Purge_waitC
		JSR	AC_loss
		BSET	6,TCR
Purge_waitC:
		BRSET	Timer2min,TFlag,Purge_waitB
		DEC	Purge_wait_cnt
		BNE	Purge_waitA
PURGE_RTN
		RTS

*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

*========================================================================*
Itotal_comp:
*NOTE: THIS SECTION KEEPS TRACK OF TOTAL AMP*HOURS AND SETS A FLAG WHEN THE
*CHARGE HAS REACHED 270 A*Hr.  THIS VALUE CAN AND WILL BE CHANGED IN THE
*FUTURE SO THAT TOTAL AMP*HOUR CHARGE CUT-OFF WILL BE DEPENDENT ON AMOUNT OF
*DISCHARGE.  THIS WILL REQUIRE TRACKING BY THE CONTROLLER AND COMMUNICATION
*BETWEEN THE TWO.  IF THE BATTERY IS ONLY DISCHARGED 100 A*Hr, THEN THE TOTAL
*AMP*HOUR CUT-OFF VALUE WILL BE 130 A*Hr (130% OF DISCHARGE).
*ITOTAL IS THE MSB OF A 32 BIT REGISTER. IF A RESOLUTION OF 10 A*Hr IS USED,
*A LOOK-UP TABLE
*Itotal
*Itotal_hi
*Itotal_mi
*Itotal_lo
*Ibatt
*
*
		BCLR	Charge_comp,IFlag
		LDA	Ibatt
		ADD	Itotal_lo
		STA	Itotal_lo
		LDA	Itotal_mi
		ADC	#$00
		STA	Itotal_mi
		BCC	CHECK_CARRY	;
		BSET	CARRY,MAINT_FLG ;
CHECK_CARRY	LDA	Itotal_hi
		ADC	#$00
		STA	Itotal_hi
*
*** NEW SECTION FOR MAINTAINANCE - THIS SECTION KEEPS TRACK OF TIME DURING
*   MAINT CHARGE (ie IN BETWEEN 30 MINUTE INTERVALS)
*
		BRCLR	MAINT,MAINT_FLG,ITOTL_COMP	;BRANCH IF NOT MAINT
		BRCLR	CARRY,MAINT_FLG,ITOTL_COMP	;
		BCLR	CARRY,MAINT_FLG ;
		INC	MAINT_AHR_CTR	;
		LDA	MAINT_AHR_CTR	;
		CMP	#$12		;IMPLIES 1 MIN AT 40 A, 2 MIN AT 20A
		BLO	ITOTL_COMP	;
		LDA	MAINT_TMR	;
		CMP	#$3C		;60 MINUTES
		BLO	MAINT_TM	;
		LDA	ICMD
		BEQ	ITOTL_COMP	; SKIP NEXT SECTION IF ICMD = ZERO
		CLR	MAINT_TMR	;RESTART TIMER
		INC	MAINT_HRS	;
MAINT_TM	CLR	MAINT_AHR_CTR
		INC	MAINT_TMR	;
		LDA	ICMD		;GET CURRENT
		CMP	#$C8		;COMPARE TO 40 A
		BHS	ITOTL_COMP	;BRANCH IF 1 MIN AT 40 AMPS
		INC	MAINT_TMR	;2 MINUTES AT 20 AMPS
		CMP	#$64		;
		BHS	ITOTL_COMP	;
		INC	MAINT_TMR
		INC	MAINT_TMR	;4 MINUTES AT 10 AMPS
*
* NEW SECTION ADDS RESOLUTION TO TOTAL AMP HOUR COUNTER
*- EACH INCREMENT OF ITOTAL = 1.25 A*HRS
*
ITOTL_COMP:
	LDA	ITOTAL_HI	;
	CMP	#$22		;
	BLO	ITOTAL_CNT	;
	CLR	ITOTAL_HI	; CLEAR ITOTAL_HI
	INC	ITOTAL		; INCREMENT TOTAL COUNTER
*
ITOTAL_CNT
	LDA	ITOTAL	;
	SUB	#ITOTAL_MAX	;GET LOOK-UP VALUE. ACC-M = ACC
*					;1.25 A*Hr PER COUNT
	BLO	Itotal_MAINT	;BRANCH IF ITOTAL < ITOTAL_MAX
	BSET	Charge_comp,IFlag	;WHEN ITOTAL = ITOTAL_MAX
Itotal_MAINT:
	BRCLR	MAINT,MAINT_FLG,ITOTAL_CNTA ; BRANCH IF NOT MAINT CHARGE
	LDA	ITOTAL		;
	CMP	#STP_MAINT_AHR	; 20*1.25 = 25 A*HRS MAX ON MAINT
*				; PER TOP OFF CHARGE
	BLS	ITOTAL_CNTA	;
	BSET	CHARGE_COMP,IFLAG	;
	INC	STP_AHR_CTR	;
	LDA	STP_AHR_CTR	;
	CMP	#$03		; REPEAT ONLY 25 A*HR CYCLE ONLY 3 TIMES
	BLS	ITOTAL_CNTA	;
	BSET	STP_MAINT,MAINT_FLG	; END MAINTAINANCE CYCLE
ITOTAL_CNTA:
		RTS

*========================================================================*
*5 MAY 93 - CHARGE CURRENT (PLMA DUTY CYCLE) INCREASED 10%. CURRENT WAS TO
*	LOW PREVIOUSLY. NOTE: THIS WILL CHANGE WITH NEW R04 BC BOARD
* 8 JUL 93 - MODIFIED SO THAT MAXIMUM VOLTAGE AND
* 5 AUG 93 - REMOVED HYSTERISIS AND CHANGED CHARGE PROFILE
*
Icmd_compute:
		BRSET	LOW_I_CHG,IFLAG,ICMD_COMPUTE_1	;
		LDA	#CURRENT_HI	;
		STA	ICMD		;
		BRA	ICMD_COMPUTE_C	; HIGH CHARGE CURRENT

*					CMP	#$78	;240 V
ICMD_COMPUTE_1:
		LDA	CURRENT_LO	;
		STA	ICMD		;
		LDA	ITOTAL		; LOAD MSB OF AMP*HOUR CHARGE
		SUB	IHIGH_END	; SUBTRACT AMP*HRS FROM HIGH I CHARGE
		CMP	ILOW_TOTL	; COMPARE TO LOW CURRENT END OF CHARG
		BLO	ICMD_END	;
		BSET	LOW_END,NIFE_FLG ; SET END OF CHARGE FLAG
		BRA	ICMD_END	;
ICMD_END:
*
Icmd_compute_C:
		LDA	Soft_start_sc	;BRANCH IF SOFT START NOT REQUESTED
		CMP	#$FF
		BEQ	Icmd_compute_E
		LDX	Icmd
		MUL
		ADD	#$80
		TXA
		ADC	#$00
		STA	Icmd
*UPDATE SOFT START SCALE FACTOR
		LDA	Soft_start_sc		;is it 0?
		BNE	Icmd_compute_D		;if yes
		CLR	Soft_start_cnt		;clear soft start counter
		INC	Soft_start_sc		;increment scale factor
Icmd_compute_D:
		INC	Soft_start_cnt		;if not
		LDA	Soft_start_cnt
		CMP	#$06
		BLO	Icmd_compute_E
		CLR	Soft_start_cnt
		INC	Soft_start_sc		;increment scale factor
Icmd_compute_E:
		RTS


*!!!!!!!!!!!!!!!!!!! ANALOG TO DIGITAL CONVERSION S/W SUPPORT !!!!!!!!!!!!!!

*========================================================================*
Vbatt_read:
		BSET	0,PORTB			;set MUX CH 01
		BCLR	1,PORTB
		AtoD	21
		FILTER	Vl,Vh
*** 1 MAY 93 THIS SECTION HAS BEEN MODIFIED TO ACCOMODATE LOWER CHARGE
* COMPLETE VOLTAGES FOR HIGHER BATTERY TEMPERATURES.
		LDA	Vh
		LDX	#$7D
		MUL				;scaling
		ADD	#$80			 ;rounding
		TXA
		ADC	#$00
		STA	Vbatt
		CMP	VBATT_TEMP_MAX		; TEST FOR VOLTAGE THRESHOLD
		BLO	Vbatt_read_A		;If yes
		INC	BATT_VOLT_MAX		;
		LDA	BATT_VOLT_MAX		;
		CMP	!10			;IF BATT > 249 FOR 10 SAMPLES
		BHS	VBATT_READ_1		;END CHARGE
		BRA	VBATT_READ_B		;
VBATT_READ_1:	BSET	Vbatt_max,IFlag		;then set V=MAX BASED ON
*						;TEMPERATURE flag to 1
*ADD SECTION TO CHECK FOR 249 VOLTS TWICE BEFORE STOPING
		BRA	Vbatt_read_B
Vbatt_read_A:
		BCLR	Vbatt_max,IFlag		;
		CLR	BATT_VOLT_MAX		;
Vbatt_read_B:
		RTS

*========================================================================*
*THIS SUB-ROUTINE IS CALLED IN THE MAIN LOOP
*
Ibatt_read:	;30 APR 93 - THIS SECTION HAS BEEN TEMPORARILY MODIFIED SO
*		; THAT Icmd IS PUT INTO IBATT INSTEAD OF THE ACTUAL VALUE
*		; SINCE HARDWARE DOES NOT SUPPORT AN6 AT THIS TIME.
*		; THIS ALLOWS THE CHARGE CYCLE TO STOP AT A DEFAULT OF 270
*		; A*Hrs REGARDLESS OF BATTERY VOLTAGE.	THIS ROUTINE DOES
*		; NOT ACCOUNT FOR BATTERY STATE OF CHARGE, THEREFORE IT WILL
*		; PROBABLY ONLY WORK WHEN THE BATTERY IS ALMOST COMPLETELY
*		; DISCHARGED
*		AtoD	26
*		LDA	AD_RR
*		LDX	#$C9
*		MUL				;scaling
*		ADD	#$80			;rounding
*		TXA
*		ADC	#$00
*		STA	Ibatt
*		RTS
		RTS
*========================================================================*
*1 MAY 93 - PROGRAM HAS BEEN MODIFIED TO CHANGE THE MAXIMUM CHARGE VOLTAGE
*AND CURRENT AS A FUNCTION OF TEMPERATURE.
*NOTE: BATTERY THERMISTOR HAS A NEGATIVE TEMPERATURE COEFFICIENT.  THIS MEANS
*THE ACTUAL VALUE IS HIGHER THAN THRESHOLD INITIALLY AND DECREASES AS IT HEAT
*UP. WHEN ACTUAL = THRESHOLD, CHARGE WILL BE STOPPED AND HIGHER VALUE IS
*LOADED INTO THRESHOLD SO THAT THE BATTERY TEMPERATURE MUST DECREASE MORE
*THAN THE PREVIOUS THRESHOLD TO START CHARGING AGAIN (SOFTSTART IS INITIATED)
*
*BATT_TEMP_LO = #$29 = 0.804 VOLTS, T = 67.5 DEGREES CELSIUS
*BATT_TEMP_HI = #$30 = 0.941 VOLTS, T = 62.0 DEGREES CELSIUS
*
** 22 AUG 93 - TEMPERATURES ABOVE WERE CHANGED.
*  THE MAXIMUM TEMP DURING CHARGING IS 50 DEGREES CELSIUS
*  MAX STARTING TEMP IS 45 DEGREES CELSIUS (AFTER WAITING 4 HOURS)
*  CHARGING STARTS IMMEDIATELY IF BATTERY TEMP IS BELOW 35 DEGREES CELSIUS.
*
Batt_temp_read:
	BCLR	0,PORTB			;set MUX CH 00
	BCLR	1,PORTB
	AtoD	21
	FILTER	Tl,Th
	STA	Batt_temp		;ACTUAL BATTERY TEMP
	CMP	Batt_temp_thr		;SEE IF BATTERY TEMP IS TO HIGH
	BHI	Batt_temp_C		;BRANCH IF NOT TOO HOT,
	BSET	BATT_HOT,IFLAG		;temperature is over
	LDA	#Batt_temp_hi		;
	STA	Batt_temp_thr		;HYSTERESIS
	BSET	Over_temp_ss,IFlag	;FORCE SOFTSTART WHEN TEMP COMES DOWN
	JMP	Batt_temp_E
Batt_temp_C:					;temperature is o'k
	LDA	#Batt_temp_lo		;RESET
	STA	Batt_temp_thr
	BRCLR	Over_temp_ss,IFlag,Batt_temp_D
	LDA	#$00			;it is first time when o'k temp
	STA	Soft_start_sc
Batt_temp_D
	BCLR	BATT_HOT,IFLAG	; BATTERY NOT HOT
		BCLR	Over_temp_ss,IFlag
Batt_temp_E:
*
*** BEGINNING OF NEW (5 JUL 93) SOFTWARE FOR TEMP VS CURRENT AND VOLTAGE
*
* TEMP VS HEX VALUE: SEE BATT_MUX.TXT FILE FOR ENTIRE LIST
*
	CLR	B_TEMP_PTR	; CLEAR POINTER FOR VOLTAGE AND CURRENT TBL
	CLR	BATT_TEMP_CMP	; CLEAR TEMPORARY COMPARISON REGISTER
	DEC	BATT_TEMP_CMP	; FORCE TO $FF
CONV_TEMP:
	LDA	BATT_TEMP	; LOAD BATTERY TEMPERATURE
	CMP	BATT_TEMP_CMP	; COMPARE TO REFERENCE TO DETERMINE WHICH
	BHS	BATT_PROF	; VALUE IN TABLE TO USE
	INC	B_TEMP_PTR	; POINT TO NEXT VALUE IN TABLE
	LDA	BATT_TEMP_CMP	; LOAD REFERENCE
	SUB	#$0A		; RESOLUTION OF BATTERY TEMPERATURE IS #$0A
	BCS	BATT_PROF	; ALL COMPARISONS EXHAUSTED IF SET
	STA	BATT_TEMP_CMP	;
	BRA	CONV_TEMP	; CONTINUE TO COMPARE
*
BATT_PROF:
	LDX	B_TEMP_PTR	; LOAD BATTERY TEMPERATURE POINTER
	LDA	BATT_TBL_V,X	; GO TO LOOK-UP TABLE, LOAD
	STA	VBATT_TEMP_MAX	; STORE NEW MAXIMUM VOLTAGE
	LDA	BATT_TBL_I,X	; GO TO LOOK-UP TABLE, LOAD
	STA	CURRENT_LO	; STORE NEW CUTBACK CURRENT
*

BATT_TMP_END	RTS	; RETURN FROM SUBROUTINE
*
***** CHARGER CONTROL TEMPERATURE *** - ADDED 6 MAY 93
*
* THIS SECTION READS TEMPERATURE AT THE THERMISTOR MOUNTED ON THE CONTROL
* ALUMINUM HEATSINK.  IF THE TEMPERATURE GETS TO HIGH, CHARGE CURRENT IS
* IS LOWERED.  IF THE TEMPERATURE CONTINUES TO INCREASE AND EXCEEDS THE
* MAXIMUM, CHARGE CURRENT IS REDUCED TO ZERO UNTIL THE TEMPERATURE DECREASES.
* HYSTERYSIS IS BUILT INTO SOFTWARE.
* THERMISTOR IS A POSITIVE TEMPERATURE COEFFICIENT DEVICE
*	A) AS TEMPERATURE INCREASES, RESISTANCE INCREASES
*	B) AS RESISTANCE INCREASES, VOLTAGE AT AtoD INCREASES
*	C) ACTUAL CHGR_TEMP REGISTER INCREASES WITH TEMPERATURE
*
*CHGR_TEMP_HI - TEMPERATURE AT WHICH CURRENT CUTS BACK
*CHGR_HYST_LO - TEMPERATURE AT WHICH CHARGER RETURNS TO NORMAL CURRENT
*CHGR_HYST_HI - TEMPERATURE AT WHICH CHARGER INITIAL GOES INTO CUT-BACK
*CHGR_TEMP_MAX- TEMPERATURE AT WHICH CHARGER CURRENT GOES TO ZERO
*CHGR_FLAG    - FLAG TO BEGIN SOFT-START CURRENT
*
CHGR_TEMP:
	BSET	0,PORTB		;set MUX CH 3
	BSET	1,PORTB
	AtoD	21		;READ CHARGER TEMPERATURE
	FILTER	Cl,Ch
	STA	CHGR_TEMP_VAL	;ACTUAL BATTERY TEMP
	CMP	CHGR_TEMP_MAX	;COMPARE TO MAXIMUM ALLOWED TEMP
	BHS	CHGR_TEMP_B	;
	LDA	CHGR_TEMP_VAL	;
	CMP	CHGR_TEMP_HI	;SEE IF CHARGER TEMPERATURE IS HIGH
	BLO	CHGR_TEMP_C	;BRANCH IF NOT TOO HOT
	LDA	#CHGR_HYST_LO	;LOAD TEMPERATURE HYSTERESIS
	STA	CHGR_TEMP_HI	;RESET HI TEMPERATURE FOR CHARGER
	BSET	CHGR_CUTBK,IFLAG	; CUT BACK CURRENT
	JMP	CHGR_TEMP_E
CHGR_TEMP_B:
	BSET	CHGR_SHUTDWN,IFLAG		; CHARGE CURRENT = 0
	LDA	#CHGR_HYST_LO	; RESET MAX TEMP FOR HYSTERESIS
	STA	CHGR_TEMP_MAX	;
	BSET	0,CHGR_TMP_FLG	; SET SOFT-START FLAG
	JMP	CHGR_TEMP_E	; CONTINUE
CHGR_TEMP_C:
	LDA	#CHGR_HYST_HI	; LOAD NORMAL TEMP CUTBACK VALUE
	STA	CHGR_TEMP_HI
	LDA	#CHGR_MAX	;
	STA	CHGR_TEMP_MAX	; RESET MAXIMUM TEMP
	BRCLR	0,CHGR_TMP_FLG,CHGR_TEMP_D	;
	LDA	#$00			;SLOWLY RAMP CURRENT AFTER TEMP -
	STA	Soft_start_sc	;RETURNS TO AN ACCEPTABLE LEVEL
CHGR_TEMP_D
	BCLR	CHGR_SHUTDWN,IFLAG    ; CHARGER NOT HOT
	BCLR	CHGR_CUTBK,IFLAG	; CHARGER NOT HOT
	BCLR	0,CHGR_TMP_FLG	;CLEAR FLAG
CHGR_TEMP_E:
	RTS
*
*========================================================================*
Vin_read:
		BSET	0,PORTB		 ;set MUX CH 1 (AC_SYNC)
		BCLR	1,PORTB
		LDA	#$20
		STA	AD_CR
		lda	#$20
Vin_read_t:
		deca
		bne	Vin_read_t
*		BRCLR	7,AD_CR,*		  ;loop until A/D completed
		LDA	AD_RR
		RTS

AC_start:
		JSR	Pwr_Src_read
		LDA	Pwr_Src
		BNE	AC_startC
AC_startA:
		JSR	WDT_pulse
		JSR	Vin_read
		CMP	#$88
		BLO	AC_startA
AC_startB:
		JSR	WDT_pulse
		JSR	Vin_read
		CMP	#$78
		BHI	AC_startB
AC_startC:
		RTS

*========================================================================*
AC_read:
		BCLR	0,PORTB			;set MUX CH 0
		BCLR	1,PORTB
		AtoD	20
		FILTER	ACl,ACh
		CMP	#$80
		BHI	AC_C			;is it higher
		BSET	0,AC_present		;no,AC is present
		RTS
AC_C:
		CLR	AC_present		;yes, AC_present flag to 0
		RTS

*========================================================================*
GWIT_in_read:
		CLR	GWIT_cnt
		CLR	GWIT_in
GWIT_in_A:
		BCLR	0,PORTB			;set MUX CH 2
		BSET	1,PORTB
		AtoD	20
		LDA	AD_RR
		CMP	#$80
		BLO	GWIT_in_C		;is it lower?
		INC	GWIT_in			;no, GWIT_in is true
GWIT_in_C:
		INC	GWIT_cnt
		LDA	GWIT_cnt
		CMP	#$08			;is it less?
		BLO	GWIT_in_A		;yes, keep reading

		LDA	GWIT_in
		CMP	#$04
		BHI	GWIT_inB		;is it higher
GWIT_inA:					;no, GWIT is lost
		CLR	GWIT_in			;reset GWIT_in flag to 0
		BRA	GWIT_inC
GWIT_inB:
		CLR	GWIT_in
		BSET	0,GWIT_in		;yes, GWIT is present
GWIT_inC:
		CLR	GWIT_cnt		;clear counter of GWIT_in
		RTS

*========================================================================*
Pwr_Src_read:
		CLR	Pwr_Src_cnt
		CLR	Pwr_Src
Pwr_Src_A:
		BSET	0,PORTB			;set MUX CH 3
		BSET	1,PORTB
		LDA	#$20
		STA	AD_CR
		LDA	#$18
Pwr_Src_B:
		DECA
		BNE	Pwr_Src_B
		BRCLR	7,AD_CR,*		;loop until conv. completed
		LDA	AD_RR
		CMP	#$80
		BLO	Pwr_Src_C		;is it lower?
		INC	Pwr_Src			;no, Pwr_Src is true
Pwr_Src_C:
		INC	Pwr_Src_cnt
		LDA	Pwr_Src_cnt
		CMP	#$08			;is it less?
		BLO	Pwr_Src_A		;yes, keep reading

		LDA	Pwr_Src
		CMP	#$04
		BHI	Pwr_SrcB		;is it higher
Pwr_SrcA:					;no, Pwr_Src is lost
		CLR	Pwr_Src			;reset Pwr_Src flag to 0
		BRA	Pwr_SrcC
Pwr_SrcB:
		CLR	Pwr_Src
		BSET	0,Pwr_Src		;yes, Pwr_Src is present
Pwr_SrcC:
		CLR	Pwr_Src_cnt		;clear counter of Pwr_Src
		RTS
*
*!!!!!!!!!!!!!!!!!!!!!!!! TIMER ROUTINES !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
WDT_pulse:
		BSET	7,PORTA
		BSET	7,PORTA
		BSET	7,PORTA
		BSET	7,PORTA
		BSET	7,PORTA
		BCLR	7,PORTA			;reset wdt to 0
		RTS

*========================================================================*
Delay_100us:
		LDA	#$20			;32 times loop
L_Delay_100us:
		DECA				;1.5 usec
		BNE	L_Delay_100us		;1.5 usec
		RTS				;3.0 usec

*========================================================================*
*========================================================================*
*
*11 MAY 93 - THIS ROUTINE WAS MODIFIED TO LOOK FOR SINGLE PHASE SUPPLY
*VOLTAGE DURING THE MAIN LOOP. IF A FUSE IS OPENED OR THE CONNECTION CORD IS
*WIRED IMPROPERLY, LOW CURRENT WILL BE SELECTED.
*
*
DELAY_6MS:
		CLR	AC_SYNC_CNT	;5;CLEAR COUNTER
		LDA	#$3C		;3; 3C = 6 mSEC
		STA	TEMP1	;4;COUNTS NUMBER OF TIMES THROUGH SAMPLE LOOP
		CLR	TEMP2	;5;STORES AC_SYNC_VALUE
* NOTE: EACH LOOP CORRESPONDS TO 200 CYCLES OR 100 uSEC
*
**AC_SYNC_RD LOOP
AC_SYNC_RD:
		JSR	WDT_pulse	;42;RESET THE WATCHDOG
		BSET	0,PORTB		;5;set MUX CH 1 (AC_SYNC)
		BCLR	1,PORTB		;5;
		LDA	#$20		;3;READ AtoD PORT 0 (AN0)
		STA	AD_CR		;4;
		lda	#$04		;3;
AC_SYNC_RD1:
		deca			;3;
		bne	AC_SYNC_RD1	;3;6*4=24 CYCLES
		LDA	#$05		;3;AtoD READ TIME = 13*5=80
		STA	AC_SYNC_TMP	;4;NOTE:ONLY 32uSEC REQUIRED
*
AC_SYNC_AD:		; 65 uSEC TOTAL, 13 CYCLES PER LOOP = 6.5 uSEC
		BRSET	7,AD_CR,AC_SYNC_DLY	;5;loop until A/D completed
		DEC	AC_SYNC_TMP	;5;
		BNE	AC_SYNC_AD	;3;
		BRA	SDELAY_6MS	;BRANCH IF READING NOT TAKEN IN TIME
AC_SYNC_DLY	LDA	AD_RR		;3;
		STA	TEMP2		;4; STORE TEMP RESULT
AC_SYNC_DLYA	DEC	AC_SYNC_TMP	;5; 6.5 uSEC DELAY PER LOOP
		BEQ	SDELAY_6MS	;3; BRANCH IF AtoD FINISHED
		NOP			;2; TIME WASTER
		BRA	AC_SYNC_DLYA	;3;
*
SDELAY_6MS:
		LDA	TEMP2		;3;GET SAMPLE
		CMP	#$80		;3;SEE IF LINE IS HIGH
		BHS	A_DELAY_6MS	;3;BRANCH IF NOT SINGLE PHASE
		NOP			;2;TIME WASTER
		BRA	B_DELAY_6MS	;3;TIME WASTER
A_DELAY_6MS	INC	AC_SYNC_CNT	;5;INCREMENT COUNTER
B_DELAY_6MS	DEC	TEMP1		;5;
		BNE	AC_SYNC_RD	;3;BRANCH IF DELAY NOT FINISHED
*					;22 TOTAL
*** 11 MAY 93 - THIS SECTION DETERMINES IF POWER SOURCE IS SINGLE PHASE
*	IF SO, CHARGE CURRENT IS REDUCED BY 1/2 IN MAIN LOOP
		LDA	AC_SYNC_CNT	;3;
		CMP	#$0A		;3;
		BLO	DELAY6_CLR	;3;BRANCH IF PWR_SRC NOT SINGLE PHASE
		BSET	SINGLE_PHASE,IFLAG	;5;INDICATE SINGLE PHASE
		BRA	DELAY6_END	;3;
DELAY6_CLR	BCLR	SINGLE_PHASE,IFLAG	;5;SOURCE NOT SINGLE PHASE
		BRA	DELAY6_END	;3;TIME WASTER
*					;17 CYCLES
DELAY6_END:
		RTS	;
* THE ABOVE SUBROUTINE CAN BE BYPASSED IF IT DOESN'T WORK BY USING;
*	BRA DELAY_6MSA	;
DELAY_6MSA	;DELAY_6MS:
		LDX	#$0A			;2 cycles
A_DELAY_6MS1:	LDA	#$F9			;2 cycles
		JSR	WDT_pulse
B_DELAY_6MS1:	DECA				;3 cycles-a-loop->1494 cycles
		BNE	B_DELAY_6MS1		;3 cycles
		DECX				;3 cycles
		BNE	A_DELAY_6MS1		;3 cycles  x-loop->25 cycles
		RTS				;6 cycles  1502*128+8->200 us

*========================================================================*
Delay_500ms:
		CLR	Delay_cnt
Delay_500msA:
		JSR	WDT_pulse
*READING VBATT, BATT_TEMP, AC_PRESENT, GWIT_IN, PWR_SRC
		JSR	Vbatt_read
		JSR	Batt_temp_read
		JSR	CHGR_TEMP
		JSR	AC_read
		JSR	GWIT_in_read
		JSR	Pwr_Src_read
		INC	Delay_cnt
		LDA	Delay_cnt
		CMP	#$40
		BLO	Delay_500msA
		RTS

*========================================================================*
Delay_synch:
		CLR	Delay_cnt
Delay_synchA:
		JSR	WDT_pulse
*READING VBATT, BATT_TEMP, AC_PRESENT, GWIT_IN, PWR_SRC
		JSR	Vbatt_read
		JSR	Batt_temp_read
		JSR	CHGR_TEMP
		JSR	AC_read
		JSR	GWIT_in_read
		JSR	Pwr_Src_read
		INC	Delay_cnt
		LDA	Delay_cnt
		CMP	#$40
		BLO	Delay_synchA
		JSR	AC_start
		RTS

*========================================================================*
Delay_5sec:
		CLR	Delay_cntr
Delay_5secA:	JSR	Delay_500ms
		INC	Delay_cntr
		LDA	Delay_cntr
		CMP	#$10
		STA	PLMB
		BLO	Delay_5secA
		RTS

*========================================================================*
*COMPARE REGISTER TIMER INTERRUPT RESET
CMP_intrpt_clear:
		LDA	CTH
		DECA
		STA	CMPH1		;INHIBIT FURTHER COMPARES
		STA	CMPH2		;
		LDA	CTL		;
		DECA
		STA	CMPL1		;
		STA	CMPL2		;DITTO
		LDA	TSR		;ENABLE OC FUNCTION
*========================================================================*
Timer_ms_init:					;
		LDA	CTH			;read actual timer value
		STA	CMPH1			;and store into CMPL1 and CMPH1
		LDA	CTL			;
		STA	CMPL1
		LDA	#T_ms_update_lo ;update reg.CMP2
		ADD	CMPL1			;to provide next interrupt
		STA	Timer_Save		;after 1 ms period.
		LDA	#T_ms_update_hi
		ADC	CMPH1
		STA	CMPH1
		LDA	Timer_save
		STA	CMPL1
		LDA	TSR			;ARM OC FLAG 1
		RTS

*========================================================================*
Timer_sec_init:					;INITIALIZE TIMER FOR A
*						ONE SECOND PERIOD
		LDA	CTH			;READ COUNTER HIGH
		STA	CMPH2			;STORE IN COMPARE HIGH BYTE2
		LDA	CTL			;READ COUNTER LOW
		STA	CMPL2
		LDA	#T_sec_update_lo	 ;update reg.CMP2
		ADD	CMPL2			;to provide next interrupt
		STA	Timer_Save		;after 250 ms period.
		LDA	#T_sec_update_hi
		ADC	CMPH2
		STA	CMPH2
		LDA	Timer_save
		STA	CMPL2
		LDA	TSR			;ARM OC FLAG 2
*		BCLR	Timer_sec_flag,TFlag
		RTS

*========================================================================*
InterruptCMP:
		JSR	WDT_pulse
		JSR	InterruptParse
		RTI

*========================================================================*
InterruptCAP:
*???????????????????
		lda	#$FF
		sta	plmb


		JSR	InterruptParse
		BRA	*
		RTI

*========================================================================*
InterruptOVF:
*???????????????????
		lda	#$FF
		sta	plmb
		JSR	InterruptParse
		BRA	*
		RTI

*========================================================================*
InterruptParse:
		LDA	TSR
		STA	Timer_status

*T_ms_check:
*		THIS WAS CHANGED TO 10 mS TO ALLOW EASY TIMING OF THE MAIN
*		LOOP
*
		BRCLR	OCF1,Timer_status,L_sec_check
		LDA	#T_ms_update_lo		;update reg.CMP1
		ADD	CMPL1			;to provide next interrupt
		STA	Timer_Save		;after ? ms period.
		LDA	#T_ms_update_hi		; HI = 13  LO = 88
		ADC	CMPH1			; 1388 = 5000*2 uS/COUNT
		STA	CMPH1			; = 10 mSEC TOTAL
		LDA	Timer_save
		STA	CMPL1
		LDA	TSR			;reset CMP interrupts
		BCLR	TIMER_10MS_FLAG,TFlag	;Timeout, clear 1ms flag

T_sec_check:
*DOESN'T SEEM TO SERVE ANY PURPOSE		;To minimize the calc.time
*						;when ? ms interrupt occured
		BRSET	Timer_sec_flag,TFlag,L_sec_check
		LDA	CMPL2
		LDA	TSR			;reset CMP interrupts
		JMP	IntParseRet
L_sec_check:
		BRCLR	OCF2,Timer_status,IntParseRet
		LDA	#T_sec_update_lo	;update reg.CMP2
		ADD	CMPL2			;to provide next interrupt
		STA	Timer_Save		;after 100 ms period.
		LDA	#T_sec_update_hi
		ADC	CMPH2
		STA	CMPH2
		LDA	Timer_save
		STA	CMPL2
		LDA	TSR			;reset CMP interrupts
*One_second_timer:
		LDA	One_second_cnt
		INCA
		STA	One_second_cnt
		CMP	#$0A
		BHS	Timer1_check
		JMP	IntParseRet
Timer1_check:
		CLR	One_second_cnt
		BRCLR	Timer1_flag,TFlag,Timer2_check
		BCLR	Timer1_flag,TFlag	;time is out, set flag to 1.
Timer2_check:
		BRCLR	Timer2_flag,TFlag,Timer3_check
		INC	Timer2
		LDA	#Timer2_const
		CMP	Timer2
		BHI	Timer3_check
		CLR	Timer2
		BCLR	Timer2_flag,TFlag
Timer3_check:
		BRCLR	Timer3_flag,TFlag,Timer4_check
		INC	Timer3
		LDA	#Timer3_const
		CMP	Timer3
		BHI	Timer4_check
		CLR	Timer3
		BCLR	Timer3_flag,TFlag
Timer4_check:
		BRCLR	Timer4_flag,TFlag,Timer5_check
		INC	Timer4
		LDA	#Timer4_const
		CMP	Timer4
		BHI	Timer5_check
		CLR	Timer4
		BCLR	Timer4_flag,TFlag
Timer5_check:
		BRCLR	Timer5_flag,TFlag,Timer6_check
		INC	Timer5
		LDA	#Timer5_const
		CMP	Timer5
		BHI	Timer6_check
		CLR	Timer5
		BCLR	Timer5_flag,TFlag
Timer6_check:
		BRCLR	Timer6_flag,TFlag,IntParseRet
		INC	Timer6
		LDA	#Timer6_const
		CMP	Timer6
		BHI	IntParseRet
		CLR	Timer6
		BCLR	Timer6_flag,TFlag
IntParseRet:
		LDA	TSR
		RTS

*========================================================================*
TINT:
*???????????????????
		lda	#$FF
		sta	plmb
		RTI

*========================================================================*
SCI:
*???????????????????
		lda	#$C0
		sta	plmb
		RTI

*========================================================================*
TOV:
*???????????????????
		lda	#$FF
		sta	plmb
		RTI

*========================================================================*
TIC:
*???????????????????
		lda	#$FF
		sta	plmb
		RTI

*========================================================================*
************ IRQ **** GFI OR GWI PROBLEM
** GREEN WIRE PROBLEM CAUSES FULL INITILIZATION. ALL MAINTAINANCE FLAGS AND
*  CHARGE DELAY FLAGS ARE RESET
*
EXT_INT:
*????????????????
		LDA	#$E0
		STA	PLMB
*RESET Icmd OUTPUT TO ZERO
		LDA	#$00
		STA	PLMA
*DROP MAIN CONTACTOR: RESET MAIN AC CONTACTOR
		BCLR	Main_AC_On_bit,Main_AC_On_port
*DROP PRECHARGE RELAY: RESET PRECHARGE RELAY
		BCLR	Pre_Q_Coil_bit,Pre_Q_Coil_port
*SET EMERGENCY SHUT DOWN FLAG
		BSET	0,GFCI_emergency

Ext_intA:
		CLR	Itotal_hi
		CLR	Itotal_mi
		CLR	Itotal_lo
Ext_intB:
*CHECK AC_present
		JSR	AC_read
		BRSET	0,AC_present,Ext_intA
		LDA	Itotal_lo
		ADD	#$01
		STA	Itotal_lo
		LDA	Itotal_mi
		ADC	#$00
		STA	Itotal_mi
*???????????????????
		sta	plmb
		LDA	Itotal_hi
		ADC	#$00
		STA	Itotal_hi
		CMP	#$01
		BLO	Ext_intB
*SOFT RESET, START PROGRAMM BECAUSE NO AC DURING 2 min
		JMP	INIT0

*========================================================================*
SOFT_INT:
*???????????????????
		lda	#$a0
		sta	plmb
*TIMER INTERRUPT RESET
		JMP	CMP_intrpt_clear
*LOOP FOREVER
		BRA	*
		RTI

*
**** LOOK-UP TABLES FOR TEMPERATURE VS CURRENT AND VOLTAGE.
*
* DESCRIPTION: STARTING AT POSITION ZERO, EACH VALUE IN THE TABLE
* CORRESPONDS TO A CURRENT OR A VOLTAGE BASED ON TEMPERATURE.
* THIS TABLE WAS DESIGNED SO THAT CHARGE PROFILE CAN BE CHANGED WITHOUT
* MODIFING THE CODE.
*
** 10 JUL 93 - TABLE ADJUSTED BASED ON RECORDED DATA (SHIFTED RIGHT 3 TIMES)
*
BATT_TBL_V	EQU	$1C00	; TABLE BEGINS AT THIS ADDRESS
	ORG	BATT_TBL_V	;
* 00	 0   1	 2   3	 4   5	 6   7	 8   9	 A   B	 C   D	 E   F
  FCB	$7D,$7D,$7D,$7D,$7D,$7D,$7D,$7D,$7D,$7D,$7D,$7D,$7D,$7C,$7B,$7A
* 10	 0   1	 2   3	 4   5	 6   7	 8   9	 A   B	 C   D	 E   F
  FCB	$79,$78,$77,$76,$76,$75,$75,$75,$75,$75,$75,$75,$75,$75,$75,$75

BATT_TBL_I     EQU     $1C20   ; TABLE BEGINS AT THIS ADDRESS
	ORG	BATT_TBL_I	;
* 00	 0   1	 2   3	 4   5	 6   7	 8   9	 A   B	 C   D	 E   F
  FCB	$64,$64,$64,$64,$64,$64,$64,$64,$64,$64,$64,$64,$64,$64,$6A,$70
* 10	 0   1	 2   3	 4   5	 6   7	 8   9	 A   B	 C   D	 E   F
  FCB	$76,$7C,$82,$88,$8E,$8E,$8E,$8E,$8E,$8E,$8E,$8E,$8E,$8E,$8E,$8E
*
**** FINISH CHARGE AHRS BASED ON DATA FROM EAGLE PITCHER **
*
*	FINISH CHARGE AHRS ARE 10% HIGHER THAN THOSE GIVEN BY EAGLE PITCHER
*	THIS ALLOWS FOR A FULL CHARGE ON SYSTEMS WHOSE OVER ALL TOLERANCES
*	WOULD OTHERWISE UNDERCHARGE A BATTERY.	FOR THOSE SYSTEMS WITH A
*	PLUS TOLERANCE OF 5%, TOTAL OVERCHARGE WILL BE 10% HIGHER THAN USUAL,
*	HOWEVER THIS IS NOT A PROBLEM ACCORDING TO EAGLE PITCHER.
*
** TABLE WAS MODIFIED 25 AUG 93 PER DON WHITMAN AT EAGLE-PITCHER
AHR_FINISH	EQU	$1C40	; TABLE BEGINS AT THIS ADDRESS
	ORG	AHR_FINISH	;
**AHR TOTAL	0   10	20  30	40  50	60  70	80  90	100 110 120 130
 FCB		$03,$04,$06,$09,$0B,$0E,$11,$14,$17,$1A,$1D,$20,$23,$26
*		140 150 160 170 180 190 200 210 220 230 240 250
 FCB		$29,$2B,$2D,$2F,$30,$31,$32,$32,$31,$31,$31,$31
*		260  270
 FCB		$31,$31
*
*
**** CHARGE DELAY VERSUS BATTERY TEMPERATURE TABLE
*
CHARGE_DELAY	EQU	$1C70
	ORG	CHARGE_DELAY	;
  FCB		$78,$73,$6E,$69,$64,$5F,$5A,$55,$50,$4B,$46,$41,$3C,$37
*
  FCB		$32,$2D,$28,$23,$1E,$19,$14,$0F,$0A,$05,$00,$00,$00,$00
*
**** MCU RESET VECTORS **
*
		ORG	$1FF0		;TIMER INTER VECTOR
		FDB	TINT		;R			1ff0
		FDB	SCI		;Serial Communication	1ff2
		FDB	TOV		;Timer Overflow		1ff4
		FDB	InterruptCMP	;Timer Output Compare	1ff6
		FDB	TIC		;Timer Input Capture	1ff8
		FDB	EXT_INT		;EXTERNAL INTERUPT	1ffa IRQ
		FDB	SOFT_INT	;SOFTWARE INTERUPT	1ffc
		FDB	INITO		;INITIZLATION		1ffe
	   END
